
# q-expansion of newform 1014.2.e.g, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1014
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 1], [[0, 1, 0, 1], 1], [[0, 2, 0, -1], 1]]

hecke_ring_character_values = [[677, [1, 0, 0, 0]], [847, [0, -1, 0, 0]]]
aps_data = [[-1, 1, 0, 0], [-1, 1, 0, 0], [2, 0, 0, -1], [0, 1, -1, 0], [-3, 3, 1, -1], [0, 0, 0, 0], [0, 4, -1, 0], [0, -3, 1, 0], [1, -1, -3, 3], [1, -1, 2, -2], [-2, 0, 0, -2], [-7, 7, 2, -2], [-1, 1, 6, -6], [0, 1, -5, 0], [-3, 0, 0, -3], [-3, 0, 0, 2], [0, -8, 0, 0], [0, 4, -3, 0], [-1, 1, -7, 7], [0, -3, 1, 0], [-8, 0, 0, -1], [-6, 0, 0, -2], [5, 0, 0, 3], [6, -6, 2, -2], [0, 6, 0, 0], [5, -5, -4, 4], [-17, 0, 0, -1], [-5, 5, -3, 3], [-2, 0, 0, -2], [0, 10, 5, 0], [-4, 4, 8, -8], [10, 0, 0, 2], [0, -5, 4, 0], [0, -4, 8, 0], [0, -8, 3, 0], [5, 0, 0, -1], [-8, 0, 0, -9], [0, 10, -2, 0], [6, -6, 2, -2], [0, 6, 6, 0], [-3, 3, 11, -11], [14, 0, 0, -3], [0, 0, -4, 0], [10, -10, 1, -1], [4, -4, 8, -8], [0, 9, -3, 0], [-6, 6, -6, 6], [20, -20, 4, -4], [0, 7, -3, 0], [2, 0, 0, 8], [6, 0, 0, 8], [-1, 0, 0, -5], [0, 2, 9, 0], [0, -10, 2, 0], [-18, 18, 5, -5], [-9, 9, 11, -11], [0, -2, 2, 0], [8, -8, 8, -8], [0, 4, -1, 0], [-5, 0, 0, -10], [-17, 17, -5, 5], [0, -8, 13, 0], [-7, 0, 0, 9], [7, 0, 0, 5], [10, 0, 0, 2], [12, 0, 0, -5], [0, -20, 0, 0], [-7, 0, 0, 8], [0, -21, 7, 0], [2, -2, -10, 10], [1, -1, 12, -12], [11, 0, 0, 7], [13, -13, -1, 1], [0, 12, -1, 0], [2, -2, 2, -2], [0, -2, -2, 0], [-9, 0, 0, -12], [0, -10, 6, 0], [-15, 15, 4, -4], [0, -16, 1, 0], [14, -14, 2, -2], [5, 0, 0, 16], [-1, 1, -7, 7], [0, 1, 8, 0], [9, -9, -5, 5], [26, 0, 0, 6], [0, 6, 10, 0], [-10, 10, 5, -5], [0, -24, -1, 0], [-9, 0, 0, -11], [3, 0, 0, -9], [-30, 30, -2, 2], [0, -9, -7, 0], [-7, 7, -1, 1], [32, 0, 0, 0], [0, -15, 15, 0], [-12, 12, -1, 1], [12, 0, 0, -17], [-7, 7, 17, -17], [23, 0, 0, 10], [1, 0, 0, 3], [-6, 6, -21, 21], [0, -28, 4, 0], [4, -4, -20, 20], [-5, 0, 0, -11], [2, 0, 0, -1], [-16, 16, 0, 0], [-23, 0, 0, -8], [-6, 0, 0, -2], [-1, 1, -4, 4], [0, 20, -12, 0], [-1, 1, 6, -6], [0, -11, -14, 0], [14, 0, 0, -2], [0, -20, -16, 0], [0, 0, -15, 0], [0, 0, -8, 0], [-2, 2, 14, -14], [14, -14, 2, -2], [0, -14, -14, 0], [-1, 1, -6, 6], [-21, 21, -4, 4], [-42, 0, 0, 2], [0, 24, 8, 0], [-35, 35, -5, 5], [14, 0, 0, 10], [0, -27, -8, 0], [0, 26, -2, 0], [-5, 0, 0, 15], [19, 0, 0, 0], [4, -4, -4, 4], [38, -38, 6, -6], [17, -17, 19, -19], [-14, 14, -4, 4], [0, 6, 10, 0], [30, -30, 8, -8], [0, 42, -4, 0], [0, -22, -10, 0], [0, 34, 0, 0], [38, -38, 9, -9], [-14, 0, 0, -18], [-6, 6, 4, -4], [0, 0, 24, 0], [30, 0, 0, 2], [0, 0, 7, -7], [0, -28, 8, 0], [-23, 0, 0, 18], [10, 0, 0, -17], [-35, 0, 0, 7], [-5, 0, 0, 7], [0, -17, 4, 0], [-12, 12, 23, -23], [36, 0, 0, -4], [-24, 24, -8, 8], [-26, 26, -6, 6], [-6, 0, 0, -2], [0, 6, 30, 0], [0, 15, -14, 0], [19, 0, 0, 14], [-22, 0, 0, -20], [-28, 28, 4, -4], [0, 0, 24, 0], [-31, 0, 0, 7], [0, -6, 14, 0], [-39, 39, -4, 4], [0, 0, 0, -12], [-7, 7, -9, 9], [0, -30, -5, 0], [10, 0, 0, -16], [-55, 0, 0, -4], [21, 0, 0, 7], [0, 3, -20, 0], [0, -26, -2, 0], [0, 28, -1, 0], [-36, 0, 0, 4], [0, 48, -5, 0], [-1, 1, -19, 19], [20, 0, 0, 19], [12, -12, 4, -4], [-2, 2, -20, 20], [5, 0, 0, -1], [-2, 0, 0, 6], [-36, 0, 0, -3], [-26, 0, 0, -8], [15, -15, -23, 23], [0, 33, -10, 0], [-46, 0, 0, 8], [-26, 0, 0, 18], [-42, 42, 1, -1], [0, -19, -19, 0], [0, 5, -6, 0], [0, -49, 9, 0], [23, 0, 0, -11], [-44, 44, 8, -8], [0, -15, 17, 0], [-14, 14, 4, -4], [-38, 0, 0, 15], [0, 0, -15, 0], [5, 0, 0, -14], [-12, 0, 0, 16], [0, 18, -13, 0], [0, 20, -12, 0], [33, -33, 4, -4], [23, 0, 0, 2], [3, -3, 19, -19], [-29, 29, -10, 10], [-33, 0, 0, 13], [0, -12, 32, 0], [-21, 21, -2, 2], [0, 16, -24, 0], [-31, 31, -8, 8], [18, 0, 0, -22], [-17, 17, 13, -13], [0, -31, -9, 0], [0, 26, 18, 0], [4, 0, 0, -11], [-20, 0, 0, -12], [0, -24, -23, 0], [0, 0, -4, 4], [-22, 0, 0, 28], [-30, 30, -13, 13], [0, 0, 0, -12], [-9, 0, 0, -20], [0, 38, -18, 0], [3, -3, -23, 23], [10, 0, 0, 9], [14, -14, 15, -15], [0, -14, -14, 0], [0, -8, -4, 0], [24, 0, 0, 20], [26, -26, -19, 19], [-4, 4, 0, 0], [-10, 10, 18, -18], [12, 0, 0, 3], [-46, 0, 0, 14], [18, 0, 0, -10], [0, 6, -16, 0], [18, -18, -1, 1], [0, 2, 18, 0], [-33, 33, -9, 9], [-13, 13, -17, 17], [-13, 13, 7, -7], [0, 5, -19, 0], [-1, 1, -20, 20], [0, -9, -20, 0], [10, 0, 0, -30], [0, 40, 12, 0], [14, -14, 26, -26], [0, -35, -7, 0], [-37, 37, -13, 13], [-30, 30, -2, 2], [-30, 30, -16, 16], [-20, 0, 0, -4], [-49, 49, 16, -16], [-18, 0, 0, 8], [0, 9, 27, 0], [0, -2, -24, 0], [-23, 23, 31, -31], [-26, 0, 0, 26], [0, 16, 9, 0], [24, 0, 0, -20], [0, 0, 12, -12], [-49, 0, 0, -4], [40, -40, 15, -15], [0, -39, 2, 0], [0, 1, 19, 0], [0, 20, -16, 0], [47, 0, 0, -6], [0, 11, -5, 0], [0, 59, -10, 0], [-8, 0, 0, -15], [8, 0, 0, 20], [-2, 2, -9, 9], [0, -4, -16, 0], [0, 50, 2, 0], [29, -29, 5, -5], [0, -16, -12, 0], [13, 0, 0, 14], [0, -10, 19, 0], [0, 3, -37, 0], [31, -31, 27, -27], [-10, 10, 6, -6], [-8, 0, 0, 4], [7, -7, -22, 22], [-17, 0, 0, -13], [19, 0, 0, 21], [3, 0, 0, -16], [-14, 0, 0, 21], [0, 5, 39, 0], [0, 20, 3, 0], [30, -30, -2, 2], [0, 71, -9, 0], [63, -63, -10, 10], [0, 6, 10, 0], [0, -42, -8, 0], [-17, 0, 0, -14], [-12, 0, 0, -36], [27, -27, 10, -10], [64, -64, -4, 4], [39, -39, -3, 3], [0, -3, 20, 0], [6, 0, 0, 9], [11, -11, 1, -1]]
